import os
import re
import sys

from django.core.management.commands.makemessages import Command as I18nCommand
from django.core.management.base import BaseCommand, CommandError
from django.utils.text import get_text_list
from django.utils.translation import templatize as orig_templatize

from mustachejs.loading import find, findAll, preprocess, MustacheJSTemplateNotFound
from mustachejs.preprocessors import I18nPreprocessor

class Command (I18nCommand):
    help = 'Adds the translatable strings from the js templates to the messages'

    def handle_noargs(self, *args, **options):
        import django.utils.translation
        django.utils.translation.templatize = templatize
        return super(Command, self).handle_noargs(*args, **options)
#        call_command('makemessages', *args, **options)


def templatize(src, origin=None):
    # Load all the js template files
#    import pdb; pdb.set_trace()
    paths = [os.path.abspath(path) for name, path in findAll('', '.*')]

    if origin and os.path.abspath(origin) in paths:
        with open(origin) as template_file:
            # Pick out all the internationalized strings
            content = template_file.read()
            processor = I18nPreprocessor()
            strings = set(re.findall(processor.trans_re, content))
#            import pdb; pdb.set_trace()
            translatable = '\n'.join(['_(r"{0}")'.format(
                string.replace('"', r'\"')) for string in strings
            ])
            return translatable

    else:
        return orig_templatize(src, origin)



from django import template

from ..conf import conf
from ..loading import find, findAll, preprocess, MustacheJSTemplateNotFound


register = template.Library()


class BaseMustacheNode(template.Node):
    preprocessors = conf.MUSTACHEJS_PREPROCESSORS

    def __init__(self, name_or_path, pattern=None):
        if pattern is None:
            self.name = template.Variable(name_or_path)
        else:
            self.path = template.Variable(name_or_path)
            self.pattern = template.Variable(pattern)

    def find_template_file(self, path):
        return find(path)

    def preprocess(self, content):
        return preprocess(content)

    def render(self, context):
        # If this is a path and a patter, loop through and render all the files
        # that match the pattern.
        if hasattr(self, 'pattern'):
            resolved_path = self.path.resolve(context)
            pattern = self.pattern.resolve(context)
            pairs = findAll(resolved_path, pattern)
            return ''.join([self.render_file(context, name, filepath)
                            for (name, filepath) in pairs])

        # If this is just a file name, render that file.
        else:
            name = self.name.resolve(context)
            return self.render_file(context, name)

    def render_file(self, context, resolved_name, filepath=None):
        try:
            if filepath is None:
                filepath = self.find_template_file(resolved_name)
            content = self.read_template_file_contents(filepath)
            content = self.preprocess(content)
            output = self.generate_node_text(resolved_name, content)

        except (IOError, MustacheJSTemplateNotFound):
            output = ""
            if conf.DEBUG:
                raise

        return output

    def read_template_file_contents(self, filepath):
        with open(filepath, "r") as fp:
            template_text = fp.read().decode(conf.FILE_CHARSET)
            template_text = self.preprocess(template_text)
            return template_text

    def generate_node_text(self, resolved_name, file_content):
        raise NotImplementedError()



import os
import re
import sys

from django.core.management.commands.makemessages import Command as I18nCommand
from django.core.management.base import BaseCommand, CommandError
from django.utils.text import get_text_list
from django.utils.translation import templatize as orig_templatize

from mustachejs.loading import find, findAll, preprocess, MustacheJSTemplateNotFound
from mustachejs.preprocessors import I18nPreprocessor

class Command (I18nCommand):
    help = 'Adds the translatable strings from the js templates to the messages'

    def handle_noargs(self, *args, **options):
        # Monkey-patch django.utils.translation.templatize to use the
        # mustachejs version when dealing with a file that is in a mustachejs
        # finder directory.
        import django.utils.translation
        django.utils.translation.templatize = templatize

        return super(Command, self).handle_noargs(*args, **options)


def templatize(src, origin=None):
    # Get all the paths that we know about
    paths = [os.path.abspath(path) for name, path in findAll('', '.*')]

    # Hijack the process if the file we're talking about is in one of the
    # finder paths.
    if origin and os.path.abspath(origin) in paths:
        with open(origin) as template_file:

            # Load the template content.
            content = template_file.read()

            # Find all the translatable strings.
            processor = I18nPreprocessor()
            strings = set(re.findall(processor.trans_re, content))

            # Build a string that looks like a Python file that's ready to be
            # translated.
            translatable = '\n'.join(['_(r"{0}")'.format(
                string.replace('"', r'\"')).replace('\n', '"\n"') for string in strings
            ])

            return translatable

    # If the file isn't in one of our paths, then delegate to the original
    # method.
    else:
        return orig_templatize(src, origin)



import re
from django.utils.translation import ugettext
from .conf import conf

class I18nPreprocessor(object):
    @property
    def tagnames(self):
        return conf.MUSTACHEJS_I18N_TAGS

    @property
    def trans_re(self):
        # Should match strings like: {{# _ }}Hello, {{ name }}.{{/ _ }}
        tagnames = '|'.join(['(?:{0})'.format(t) for t in self.tagnames])
        start_tag = r'\{\{#\s*(?:' + tagnames + r')\s*\}\}'
        end_tag = r'\{\{\/\s*(?:' + tagnames + r')\s*\}\}'

        return start_tag + '(.*)' + end_tag

    def translate(self, match):
        """Translate a result of matching the compiled trans_re pattern."""
        string = match.group(1)
        return ugettext(string) if len(string) > 0 else u''

    def process(self, content):
        return re.sub(self.trans_re, self.translate, content, flags=re.DOTALL)



from django.core.exceptions import ImproperlyConfigured
from django.utils.importlib import import_module

from .conf import conf



def find(name):
    for finder in finders:
        filepath = finder.find(name)
        if filepath is not None:
            return filepath

    raise MustacheJSTemplateNotFound(name)


def preprocess(content):
    for preprocessor in preprocessors:
        content = preprocessor.process(content)

    return content


def findAll(dir, regex):
    result = []
    for finder in regexfinders:
        paths = finder.findAll(dir, regex)
        if paths is not None:
            result += paths
    return result


def _get_classes(dotted_paths):
    ret = []
    for path in dotted_paths:
        modpath, cls_name = path.rsplit(".", 1)
        try:
            mod = import_module(modpath)
        except ImportError, e:
            raise ImproperlyConfigured(
                "ImportError %s: %s" % (modpath, e.args[0]))

        try:
            cls = getattr(mod, cls_name)
        except AttributeError, e:
            raise ImproperlyConfigured(
                "AttributeError %s: %s" % (cls_name, e.args[0]))

        ret.append(cls())

    return ret


def _get_finders():
    return _get_classes(conf.MUSTACHEJS_FINDERS)


def _get_regexfinders():
    return _get_classes(conf.MUSTACHEJS_REGEX_FINDERS)


def _get_preprocessors():
    return _get_classes(conf.MUSTACHEJS_PREPROCESSORS)


# Instantiate finders
finders = _get_finders()
regexfinders = _get_regexfinders()
preprocessors = _get_preprocessors()


class MustacheJSTemplateNotFound(Exception):
    pass
